#!/usr/bin/perl

# MyBlaster.pl:  Controller for the MyBlaster serial IR blaster (www.mytvstore.com)
# By William Munson [wmunson] (July 26, 2004, ver-1.4)
# 
# This code is being released into the public domain for non-profit use. 
# For commercial use please contact me at our support forum:
# http://www.mythtvtalk.com/forum/viewtopic.php?p=622#622
#
# Version 1.0 - Initial release of code.
# Version 1.1 - Added macro feature.
# Version 1.2 - Fixed some timing issues in the data transfer to MyBlaster.
# Version 1.3 - Fixed output for devices other than DSS/Satellite receivers.
#             - Added support for port locking which keeps a second instance 
#                   suspended until the 1st has completed. 
# Version 1.4 - Public Release #1 - Updated documentation for release.


$|=1;
use POSIX qw(:termios_h);
use FileHandle;
use Time::HiRes qw( sleep ); 

	  	  
# Defaults to standard COM2 port, 19200 baud.
# my $serial=init_serial("/dev/ttyS0","19200"); # Com 1
my $serial=init_serial("/dev/ttyS1","19200"); # Com 2

# Device code for the piece of equipment you want to control. This info can
# be found in the MyBlaster Library folder.
#
# 775 controls a standard Dishnet receiver.
$remote_code="775";

# Device Type is a hex value from 0 to A and represents the type of device you 
# want to control. Below is a list of the available device type codes.
# 0=TV, 1=CABLE, 2=Video Acc, 3=DSS/Satellite, 4-VCR, 5=Laser Disk, 6=DVD
# 7=Tuner/Amp, 8=Amplifier, 9=CD, A=Home Control
$Device_Type="0";

# Duplicate process protection. Enable this if you use the same blaster to control
# multiple devices. With this enabled, multiple instances of the program will each
# sleep until the blaster port is free. Uses a lock file.
$Use_Locking=1;

# Lock File name.
$LockFile="/tmp/MyBlaster.lock";

# Use verbose output mode for debugging.
$verbose=0;

# Time delay between each channel digit.
$inter_key_delay=1.0;

# Emit time. This is the length of time the command is sent. Probably a
# good idea to leave this alone but if you are getting repeats of buttons
# then you can shorten it a little bit.
$Emit_Time=0.333;

# Enable to press a key to finalize channel entry
$finalize=0;

# Key used to finalize channel entry
$finalize_key="select";

# Enable to clear the on screen display more quickly. 
$quick_clear=0;

# Key used to clear the display. Taken from keymap below.
$quick_clear_key="exit";

# Time to wait until clearing the display
$clear_delay=2.0;

# Misc variables needed by the program
$Count=0;


# These keys should be common to all device codes. The remote buttons 
# may have different names. Feel free to change the name but not the value.
%keymap=(1 => "0x01",
	 2 => "0x02",
	 3 => "0x03",
	 4 => "0x04",
	 5 => "0x05",
	 6 => "0x06",
	 7 => "0x07",
	 8 => "0x08",
	 9 => "0x09",
	 0 => "0x0a",	 
    vol_up => "0x0b",
    vol_dn => "0x0c",
      mute => "0x0d",
     ch_up => "0x0e",
     ch_dn => "0x0f",
     power => "0x10",
     enter => "0x11",
    ch_100 => "0x11",
 last_chan => "0x12",
    tv_sat => "0x13",
     input => "0x13",	 
      play => "0x14",
      stop => "0x15",
   page_up => "0x16",
      ffwd => "0x16",
       rew => "0x17",
   page_dn => "0x17",
     pause => "0x18",
    record => "0x19",
      menu => "0x1a",
        up => "0x1b",
      down => "0x1c",
      left => "0x1d",
     right => "0x1e",
    select => "0x1f",
      exit => "0x20",
   display => "0x21",
     guide => "0x22");



# Start of main code.
#
# Pull the command line info into the program
$input_cmd=$ARGV[0];

if ($Use_Locking) {
	if (-e $LockFile) { #found a lock file so sleep
		sleep(5.0);
	}
	open(OUTFILE, ">$LockFile") || die "cannot open file"; # create our lock file
}
	
# These bits define which remote control protocol to use and the device type. 
#
# Convert remote code to MyBlaster format
$hex_remote_code = sprintf("%X", $remote_code);

if ($remote_code > 255) {
	$hi_bit="0x" . $Device_Type . substr($hex_remote_code, 0, 1);
	$lo_bit="0x" . substr($hex_remote_code, 1, 2);
} else {
	$hi_bit="0x" . $Device_Type . "0";
	if ($remote_code > 15) {
		$lo_bit="0x" . substr($hex_remote_code, 0, 2);
	} else {
		$lo_bit="0x0" . substr($hex_remote_code, 0, 1);
	}	
}
print "hi bit = " if ($verbose);
print $hi_bit if ($verbose);
print "\n" if ($verbose);
print "lo bit = " if ($verbose);
print $lo_bit if ($verbose);
print "\n" if ($verbose);


# Start of command decoding
if (length($input_cmd)) {
	# Check to see if a channel number was entered.
	if ($input_cmd != 0) {
		change_channel($input_cmd);
		if ($Use_Locking) {
			close(OUTFILE);
			unlink($LockFile);
		}
		exit; 
	}
	if ($input_cmd eq "macro") {
		# Do macro command. Use your own key sequence here. 
		# Change it around as required for your needs.
		mb_key("down");
		sleep(1.0);
		mb_key("up");
		sleep(1.0);
		mb_key("select");
		sleep(1.0);
		mb_key("display");
		if ($Use_Locking) {
			close(OUTFILE);
			unlink($LockFile);
		}
		exit;
	}
	# No special processing required. Send a single command.
	mb_key($input_cmd);
	if ($Use_Locking) {
		close(OUTFILE);
		unlink($LockFile);
	}
} 	


# begin subroutines.

sub change_channel {
	# Send a command for each digit of the channel number
	my ($channel_num)=@_;
	my ($position)=0;
	my ($digit)="";
	
	if ($verbose){
		print "Channel number: "; 
		print $channel_num;
		print "\n";
	}
	while ($position<length($channel_num)){
		$digit=substr($channel_num,$position,1);
		if ($verbose){
			print "Digit: ";
			print $digit;
			print "\n";
		}	
		mb_key($digit);
		$position++;
		sleep($inter_key_delay);
	}
	# wait a little bit and press enter to finalize channel entry
	if ($finalize){
		print "Finalize Entry:\n" if ($verbose);
		mb_key($finalize_key);
	}		
	
	#If enabled, clear the display after sleeping one second
	if ($quick_clear){
		sleep($clear_delay);
		print "Clear Display:\n" if ($verbose);
		mb_key($quick_clear_key);
	}	
}	


sub mb_key {
    	my ($key)=@_;
    	my ($Count)=0;
    	my ($Looping)=1;
    	return undef unless $keymap{$key};
    	simple_command("0x00");
	sleep(0.1);
	# send the data packet
	simple_command("0xBC","0xAC","0x0D","0x00","0x06","0x01",$hi_bit,$lo_bit,$keymap{$key},"0x00","0x00");
	sleep($Emit_Time);	#Execution delay
	simple_command("0x00");	
	print "\n" if ($verbose);
	get_reply();
}

sub simple_command {
    	if (defined(sendbytes(@_))) {	
		return(1);
    	} else {
		return(undef);
    	}
}

sub mb_command {
    	sendbytes(@_);
}


sub sendbytes {
    	(@send)=@_;
    	foreach (@send) { s/^0x//g; $_=hex($_); }
    	print "SEND: " if ($verbose);
    	foreach $num (@send) {
		$str=pack('C',$num);
		printf("0x%X ", $num, $str) if ($verbose);
      		syswrite($serial,$str,length($str));
    	}
    	print "\n" if ($verbose);
}

sub get_reply {
    	my $starttime=time();
    	my ($last,$ok,@ret);
	my ($Expected_Count)=@_;
	my ($Count)=0;
    	print "RECV: " if ($verbose);
	
    	while (1) {       
       		$ret=sysread($serial,$buf,1);
       		$str=sprintf("0x%2.2X", ord($buf));       

       		# busy wait bad!
       		die ("\n") if (time() - $starttime > 1);
       		next if $str eq "0x00";
       		$Count++;
       		if ($pkt_decode{$str}) {
	   		print $str if ($verbose);
	   		print "[$pkt_decode{$str}] " if ($verbose);
       		} else {
	   		$_=$str; s/^0x//g; $_=hex($_);  
	   		printf("$str(%3.3s) ",$_) if ($verbose);
	   		push (@ret,$_); 
       		}
       
       		$ok=1 if ($terminal{$str} > 0);
       		last if $str eq "0x31";
       		last if $str eq "0x01";
       		next;
   	}
   	print "\n\n" if ($verbose);
    
   	return @ret if ($ok);
   	return undef;
}


sub init_serial {
    	my($port,$baud)=@_;
    	my($termios,$cflag,$lflag,$iflag,$oflag);
    	my($voice);
 
 	my $serial=new FileHandle("+>$port") || die "Could not open $port: $!\n";
    
    	$termios = POSIX::Termios->new();
    	$termios->getattr($serial->fileno()) || die "getattr: $!\n";
    	$cflag= 0 | CS8 | HUPCL | CREAD | CLOCAL;
    	$lflag= 0;
    	$iflag= 0 | IGNBRK | IGNPAR | IXON | IXOFF | IGNCR;
    	#$iflag= 0 | IGNBRK | IGNPAR | IGNCR;
    	$oflag= 0;
        
    	$termios->setcflag($cflag);
    	$termios->setlflag($lflag);
    	$termios->setiflag($iflag);
    	$termios->setoflag($oflag);
    	$termios->setattr($serial->fileno(),TCSANOW) || die "setattr: $!\n";
    	eval qq[
      		\$termios->setospeed(POSIX::B$baud) || die "setospeed: \$!\n";
      		\$termios->setispeed(POSIX::B$baud) || die "setispeed: \$!\n";
    	];
    
    	die $@ if $@;
    
    	$termios->setattr($serial->fileno(),TCSANOW) || die "setattr: $!\n";

    	# This gets rid of all the special characters..
    	$termios->getattr($serial->fileno()) || die "getattr: $!\n";	
    	for (0..NCCS) {	
		if ($_ == NCCS) { last; }

		# Dont mess up XON/XOFF..
		if ($_ == VSTART || $_ == VSTOP) { next; }

		$termios->setcc($_,0);	
    	}
    	$termios->setattr($serial->fileno(),TCSANOW) || die "setattr: $!\n";
    
    	return $serial;
}



